@@ -123,7 +123,7 @@ def _create_transport(index, transport, check_native_endpoint=None):
     valid_transport_keys = [
         'type', 'url', 'endpoint', 'serializer', 'serializers', 'options',
         'max_retries', 'max_retry_delay', 'initial_retry_delay',
-        'retry_delay_growth', 'retry_delay_jitter',
+        'retry_delay_growth', 'retry_delay_jitter', 'proxy',
     ]
     for k in transport.keys():
         if k not in valid_transport_keys:
@@ -139,6 +139,23 @@ def _create_transport(index, transport, check_native_endpoint=None):
     else:
         transport['type'] = 'websocket'
 
+    if 'proxy' in transport and kind != 'websocket':
+        raise ValueError(
+            "proxy= only supported for type=websocket transports"
+        )
+    proxy = transport.get("proxy", None)
+    if proxy is not None:
+        for k in proxy.keys():
+            if k not in ['host', 'port']:
+                raise ValueError(
+                    "Unknown key '{}' in proxy config".format(k)
+                )
+        for k in ['host', 'port']:
+            if k not in proxy:
+                raise ValueError(
+                    "Proxy config requires '{}'".formaT(k)
+                )
+
     options = dict()
     if 'options' in transport:
         options = transport['options']
@@ -236,6 +253,7 @@ def _create_transport(index, transport, check_native_endpoint=None):
         url=transport.get('url', None),
         endpoint=endpoint_config,
         serializers=serializer_config,
+        proxy=proxy,
         options=options,
         **kw
     )
@@ -252,6 +270,7 @@ class _Transport(object):
                  initial_retry_delay=1.5,
                  retry_delay_growth=1.5,
                  retry_delay_jitter=0.1,
+                 proxy=None,
                  options=dict()):
         """
         """
@@ -273,6 +292,7 @@ class _Transport(object):
         self.initial_retry_delay = initial_retry_delay
         self.retry_delay_growth = retry_delay_growth
         self.retry_delay_jitter = retry_delay_jitter
+        self.proxy = proxy  # this is a dict of proxy config
 
         # used via can_reconnect() and failed() to record this
         # transport is never going to work
@@ -361,6 +381,7 @@ class Component(ObservableMixin):
             def do_subscription(session, details):
                 return session.subscribe(fn, topic=topic, options=options)
             self.on('join', do_subscription)
+            return fn
         return decorator
 
     def register(self, uri, options=None):
@@ -383,6 +404,7 @@ class Component(ObservableMixin):
             def do_registration(session, details):
                 return session.register(fn, procedure=uri, options=options)
             self.on('join', do_registration)
+            return fn
         return decorator
 
     def __init__(self, main=None, transports=None, config=None, realm=u'default', extra=None,
@@ -535,6 +557,15 @@ class Component(ObservableMixin):
         # specified in the docstring.
         self._done_f = txaio.create_future()
 
+        def _reset(arg):
+            """
+            if the _done_f future is resolved (good or bad), we want to set it
+            to None in our class
+            """
+            self._done_f = None
+            return arg
+        txaio.add_callbacks(self._done_f, _reset, _reset)
+
         # Create a generator of transports that .can_reconnect()
         transport_gen = itertools.cycle(self._transports)
 
@@ -627,7 +658,10 @@ class Component(ObservableMixin):
             def session_done(x):
                 txaio.resolve(self._done_f, None)
 
-            connect_f = self._connect_once(loop, transport_candidate[0])
+            connect_f = txaio.as_future(
+                self._connect_once,
+                loop, transport_candidate[0],
+            )
             txaio.add_callbacks(connect_f, session_done, connect_error)
 
         def transport_check(_):
@@ -788,7 +822,10 @@ class Component(ObservableMixin):
 
         transport.connect_attempts += 1
 
-        d = self._connect_transport(reactor, transport, create_session, done)
+        d = txaio.as_future(
+            self._connect_transport,
+            reactor, transport, create_session, done,
+        )
 
         def on_error(err):
             """
