@@ -41,6 +41,7 @@ __all__ = (
     'Challenge',
     'HelloDetails',
     'SessionDetails',
+    'SessionIdent',
     'CloseDetails',
     'SubscribeOptions',
     'EventDetails',
@@ -375,10 +376,129 @@ class SessionDetails(object):
         self.resumable = resumable
         self.resume_token = resume_token
 
+    def marshal(self):
+        obj = {
+            u'realm': self.realm,
+            u'session': self.session,
+            u'authid': self.authid,
+            u'authrole': self.authrole,
+            u'authmethod': self.authmethod,
+            u'authprovider': self.authprovider,
+            u'authextra': self.authextra,
+            u'resumed': self.resumed,
+            u'resumable': self.resumable,
+            u'resume_token': self.resume_token
+        }
+        return obj
+
     def __str__(self):
         return u"SessionDetails(realm=<{}>, session={}, authid=<{}>, authrole=<{}>, authmethod={}, authprovider={}, authextra={}, resumed={}, resumable={}, resume_token={})".format(self.realm, self.session, self.authid, self.authrole, self.authmethod, self.authprovider, self.authextra, self.resumed, self.resumable, self.resume_token)
 
 
+@public
+class SessionIdent(object):
+    """
+    WAMP session identification information.
+
+    A WAMP session joined on a realm on a WAMP router is identified technically
+    by its session ID (``session``) already.
+
+    The permissions the session has are tied to the WAMP authentication role (``authrole``).
+
+    The subject behind the session, eg the user or the application component is identified
+    by the WAMP authentication ID (``authid``). One session is always authenticated under/as
+    one specific ``authid``, but a given ``authid`` might have zero, one or many sessions
+    joined on a router at the same time.
+    """
+
+    __slots__ = (
+        'session',
+        'authid',
+        'authrole',
+    )
+
+    def __init__(self, session=None, authid=None, authrole=None):
+        """
+
+        :param session: WAMP session ID of the session.
+        :type session: int
+
+        :param authid: The WAMP authid of the session.
+        :type authid: str
+
+        :param authrole: The WAMP authrole of the session.
+        :type authrole: str
+        """
+        assert(session is None or type(session) in six.integer_types)
+        assert(authid is None or type(authid) == six.text_type)
+        assert(type(authrole) == six.text_type)
+
+        self.session = session
+        self.authid = authid
+        self.authrole = authrole
+
+    def __str__(self):
+        return u"SessionIdent(session={}, authid={}, authrole={})".format(self.session, self.authid, self.authrole)
+
+    def marshal(self):
+        obj = {
+            u'session': self.session,
+            u'authid': self.authid,
+            u'authrole': self.authrole,
+        }
+        return obj
+
+    @staticmethod
+    def from_calldetails(call_details):
+        """
+        Create a new session identification object from the caller information
+        in the call details provided.
+
+        :param call_details: Details of a WAMP call.
+        :type call_details: :class:`autobahn.wamp.types.CallDetails`
+
+        :returns: New session identification object.
+        :rtype: :class:`autobahn.wamp.types.SessionIdent`
+        """
+        assert isinstance(call_details, CallDetails)
+
+        if call_details.forward_for:
+            caller = call_details.forward_for[0]
+            session_ident = SessionIdent(caller['session'],
+                                         caller['authid'],
+                                         caller['authrole'])
+        else:
+            session_ident = SessionIdent(call_details.caller,
+                                         call_details.caller_authid,
+                                         call_details.caller_authrole)
+        return session_ident
+
+    @staticmethod
+    def from_eventdetails(event_details):
+        """
+        Create a new session identification object from the publisher information
+        in the event details provided.
+
+        :param event_details: Details of a WAMP event.
+        :type event_details: :class:`autobahn.wamp.types.EventDetails`
+
+        :returns: New session identification object.
+        :rtype: :class:`autobahn.wamp.types.SessionIdent`
+        """
+        assert isinstance(event_details, EventDetails)
+
+        if event_details.forward_for:
+            publisher = event_details.forward_for[0]
+            session_ident = SessionIdent(publisher['session'],
+                                         publisher['authid'],
+                                         publisher['authrole'])
+        else:
+            session_ident = SessionIdent(event_details.publisher,
+                                         event_details.publisher_authid,
+                                         event_details.publisher_authrole)
+        return session_ident
+
+
 @public
 class CloseDetails(object):
     """
@@ -409,6 +529,13 @@ class CloseDetails(object):
         self.reason = reason
         self.message = message
 
+    def marshal(self):
+        obj = {
+            u'reason': self.reason,
+            u'message': self.message
+        }
+        return obj
+
     def __str__(self):
         return u"CloseDetails(reason=<{}>, message='{}')".format(self.reason, self.message)
 
@@ -505,9 +632,11 @@ class EventDetails(object):
         'topic',
         'retained',
         'enc_algo',
+        'forward_for',
     )
 
-    def __init__(self, subscription, publication, publisher=None, publisher_authid=None, publisher_authrole=None, topic=None, retained=None, enc_algo=None):
+    def __init__(self, subscription, publication, publisher=None, publisher_authid=None, publisher_authrole=None,
+                 topic=None, retained=None, enc_algo=None, forward_for=None):
         """
 
         :param subscription: The (client side) subscription object on which this event is delivered.
@@ -538,6 +667,9 @@ class EventDetails(object):
         :param enc_algo: Payload encryption algorithm that
             was in use (currently, either ``None`` or ``u'cryptobox'``).
         :type enc_algo: str or None
+
+        :param forward_for: When this Event is forwarded for a client (or from an intermediary router).
+        :type forward_for: list[dict]
         """
         assert(isinstance(subscription, Subscription))
         assert(type(publication) in six.integer_types)
@@ -547,6 +679,13 @@ class EventDetails(object):
         assert(topic is None or type(topic) == six.text_type)
         assert(retained is None or type(retained) is bool)
         assert(enc_algo is None or type(enc_algo) == six.text_type)
+        assert(forward_for is None or type(forward_for) == list)
+        if forward_for:
+            for ff in forward_for:
+                assert type(ff) == dict
+                assert 'session' in ff and type(ff['session']) in six.integer_types
+                assert 'authid' in ff and type(ff['authid']) == six.text_type
+                assert 'authrole' in ff and type(ff['authrole']) == six.text_type
 
         self.subscription = subscription
         self.publication = publication
@@ -556,9 +695,10 @@ class EventDetails(object):
         self.topic = topic
         self.retained = retained
         self.enc_algo = enc_algo
+        self.forward_for = forward_for
 
     def __str__(self):
-        return u"EventDetails(subscription={}, publication={}, publisher={}, publisher_authid={}, publisher_authrole={}, topic=<{}>, retained={}, enc_algo={})".format(self.subscription, self.publication, self.publisher, self.publisher_authid, self.publisher_authrole, self.topic, self.retained, self.enc_algo)
+        return u"EventDetails(subscription={}, publication={}, publisher={}, publisher_authid={}, publisher_authrole={}, topic=<{}>, retained={}, enc_algo={}, forward_for={})".format(self.subscription, self.publication, self.publisher, self.publisher_authid, self.publisher_authrole, self.topic, self.retained, self.enc_algo, self.forward_for)
 
 
 @public
@@ -578,6 +718,7 @@ class PublishOptions(object):
         'eligible_authid',
         'eligible_authrole',
         'retain',
+        'forward_for',
         'correlation_id',
         'correlation_uri',
         'correlation_is_anchor',
@@ -594,6 +735,7 @@ class PublishOptions(object):
                  eligible_authid=None,
                  eligible_authrole=None,
                  retain=None,
+                 forward_for=None,
                  correlation_id=None,
                  correlation_uri=None,
                  correlation_is_anchor=None,
@@ -628,6 +770,9 @@ class PublishOptions(object):
 
         :param retain: If ``True``, request the broker retain this event.
         :type retain: bool or None
+
+        :param forward_for: When this Event is forwarded for a client (or from an intermediary router).
+        :type forward_for: list[dict]
         """
         assert(acknowledge is None or type(acknowledge) == bool)
         assert(exclude_me is None or type(exclude_me) == bool)
@@ -639,6 +784,14 @@ class PublishOptions(object):
         assert(eligible_authrole is None or type(eligible_authrole) == six.text_type or (type(eligible_authrole) == list and all(type(x) == six.text_type for x in eligible_authrole)))
         assert(retain is None or type(retain) == bool)
 
+        assert(forward_for is None or type(forward_for) == list)
+        if forward_for:
+            for ff in forward_for:
+                assert type(ff) == dict
+                assert 'session' in ff and type(ff['session']) in six.integer_types
+                assert 'authid' in ff and type(ff['authid']) == six.text_type
+                assert 'authrole' in ff and type(ff['authrole']) == six.text_type
+
         self.acknowledge = acknowledge
         self.exclude_me = exclude_me
         self.exclude = exclude
@@ -648,6 +801,7 @@ class PublishOptions(object):
         self.eligible_authid = eligible_authid
         self.eligible_authrole = eligible_authrole
         self.retain = retain
+        self.forward_for = forward_for
 
         self.correlation_id = correlation_id
         self.correlation_uri = correlation_uri
@@ -687,10 +841,13 @@ class PublishOptions(object):
         if self.retain is not None:
             options[u'retain'] = self.retain
 
+        if self.forward_for is not None:
+            options[u'forward_for'] = self.forward_for
+
         return options
 
     def __str__(self):
-        return u"PublishOptions(acknowledge={}, exclude_me={}, exclude={}, exclude_authid={}, exclude_authrole={}, eligible={}, eligible_authid={}, eligible_authrole={}, retain={})".format(self.acknowledge, self.exclude_me, self.exclude, self.exclude_authid, self.exclude_authrole, self.eligible, self.eligible_authid, self.eligible_authrole, self.retain)
+        return u"PublishOptions(acknowledge={}, exclude_me={}, exclude={}, exclude_authid={}, exclude_authrole={}, eligible={}, eligible_authid={}, eligible_authrole={}, retain={}, forward_for={})".format(self.acknowledge, self.exclude_me, self.exclude, self.exclude_authid, self.exclude_authrole, self.eligible, self.eligible_authid, self.eligible_authrole, self.retain, self.forward_for)
 
 
 @public
@@ -791,9 +948,11 @@ class CallDetails(object):
         'caller_authrole',
         'procedure',
         'enc_algo',
+        'forward_for',
     )
 
-    def __init__(self, registration, progress=None, caller=None, caller_authid=None, caller_authrole=None, procedure=None, enc_algo=None):
+    def __init__(self, registration, progress=None, caller=None, caller_authid=None,
+                 caller_authrole=None, procedure=None, enc_algo=None, forward_for=None):
         """
 
         :param registration: The (client side) registration object this invocation is delivered on.
@@ -820,6 +979,9 @@ class CallDetails(object):
         :param enc_algo: Payload encryption algorithm that
             was in use (currently, either `None` or `"cryptobox"`).
         :type enc_algo: str or None
+
+        :param forward_for: When this Call is forwarded for a client (or from an intermediary router).
+        :type forward_for: list[dict]
         """
         assert(isinstance(registration, Registration))
         assert(progress is None or callable(progress))
@@ -829,6 +991,14 @@ class CallDetails(object):
         assert(procedure is None or type(procedure) == six.text_type)
         assert(enc_algo is None or type(enc_algo) == six.text_type)
 
+        assert(forward_for is None or type(forward_for) == list)
+        if forward_for:
+            for ff in forward_for:
+                assert type(ff) == dict
+                assert 'session' in ff and type(ff['session']) in six.integer_types
+                assert 'authid' in ff and type(ff['authid']) == six.text_type
+                assert 'authrole' in ff and type(ff['authrole']) == six.text_type
+
         self.registration = registration
         self.progress = progress
         self.caller = caller
@@ -836,9 +1006,10 @@ class CallDetails(object):
         self.caller_authrole = caller_authrole
         self.procedure = procedure
         self.enc_algo = enc_algo
+        self.forward_for = forward_for
 
     def __str__(self):
-        return u"CallDetails(registration={}, progress={}, caller={}, caller_authid={}, caller_authrole={}, procedure=<{}>, enc_algo={})".format(self.registration, self.progress, self.caller, self.caller_authid, self.caller_authrole, self.procedure, self.enc_algo)
+        return u"CallDetails(registration={}, progress={}, caller={}, caller_authid={}, caller_authrole={}, procedure=<{}>, enc_algo={}, forward_for={})".format(self.registration, self.progress, self.caller, self.caller_authid, self.caller_authrole, self.procedure, self.enc_algo, self.forward_for)
 
 
 @public
@@ -850,6 +1021,7 @@ class CallOptions(object):
     __slots__ = (
         'on_progress',
         'timeout',
+        'forward_for',
         'correlation_id',
         'correlation_uri',
         'correlation_is_anchor',
@@ -859,6 +1031,7 @@ class CallOptions(object):
     def __init__(self,
                  on_progress=None,
                  timeout=None,
+                 forward_for=None,
                  correlation_id=None,
                  correlation_uri=None,
                  correlation_is_anchor=None,
@@ -871,12 +1044,24 @@ class CallOptions(object):
 
         :param timeout: Time in seconds after which the call should be automatically canceled.
         :type timeout: float
+
+        :param forward_for: When this Call is forwarded for a client (or from an intermediary router).
+        :type forward_for: list[dict]
         """
         assert(on_progress is None or callable(on_progress))
         assert(timeout is None or (type(timeout) in list(six.integer_types) + [float] and timeout > 0))
 
+        assert(forward_for is None or type(forward_for) == list)
+        if forward_for:
+            for ff in forward_for:
+                assert type(ff) == dict
+                assert 'session' in ff and type(ff['session']) in six.integer_types
+                assert 'authid' in ff and type(ff['authid']) == six.text_type
+                assert 'authrole' in ff and type(ff['authrole']) == six.text_type
+
         self.on_progress = on_progress
         self.timeout = timeout
+        self.forward_for = forward_for
 
         self.correlation_id = correlation_id
         self.correlation_uri = correlation_uri
@@ -895,10 +1080,13 @@ class CallOptions(object):
         if self.on_progress is not None:
             options[u'receive_progress'] = True
 
+        if self.forward_for is not None:
+            options[u'forward_for'] = self.forward_for
+
         return options
 
     def __str__(self):
-        return u"CallOptions(on_progress={}, timeout={})".format(self.on_progress, self.timeout)
+        return u"CallOptions(on_progress={}, timeout={}, forward_for={})".format(self.on_progress, self.timeout, self.forward_for)
 
 
 @public
@@ -912,6 +1100,7 @@ class CallResult(object):
         'results',
         'kwresults',
         'enc_algo',
+        'forward_for',
     )
 
     def __init__(self, *results, **kwresults):
@@ -926,12 +1115,23 @@ class CallResult(object):
         enc_algo = kwresults.pop('enc_algo', None)
         assert(enc_algo is None or type(enc_algo) == six.text_type)
 
+        forward_for = kwresults.pop('forward_for', None)
+        assert(forward_for is None or type(forward_for) == list)
+        if forward_for:
+            for ff in forward_for:
+                assert type(ff) == dict
+                assert 'session' in ff and type(ff['session']) in six.integer_types
+                assert 'authid' in ff and type(ff['authid']) == six.text_type
+                assert 'authrole' in ff and type(ff['authrole']) == six.text_type
+
         self.enc_algo = enc_algo
+        self.forward_for = forward_for
+
         self.results = results
         self.kwresults = kwresults
 
     def __str__(self):
-        return u"CallResult(results={}, kwresults={}, enc_algo={})".format(self.results, self.kwresults, self.enc_algo)
+        return u"CallResult(results={}, kwresults={}, enc_algo={}, forward_for={})".format(self.results, self.kwresults, self.enc_algo, self.forward_for)
 
 
 @public
